@@ -24,6 +24,7 @@
 #include <sys/time.h>
 #include <arpa/inet.h>
 #include <stdlib.h>
+#include <stdbool.h>
 #include <unistd.h>
 #include <stdio.h>
 #include <netinet/in.h>
@@ -194,6 +195,9 @@ typedef struct port_info
     /* Allow RFC 2217 mode */
     int allow_2217;
 
+    /* Send a break if we get a sync command? */
+    int telnet_brk_on_sync;
+
     /* kickolduser mode */
     int kickolduser_mode;
 
@@ -209,6 +213,20 @@ typedef struct port_info
     /* String to send to device at close, or NULL if none. */
     char *closestr;
 
+    /*
+     * Close on string to shutdown connection when received from
+     * serial side, or NULL if none.
+     */
+    char *closeon;
+    unsigned int closeon_pos;
+    unsigned int closeon_len;
+
+    /*
+     * Close the session when all the output has been written to the
+     * TCP port.
+     */
+    bool close_on_output_done;
+
     /*
      * File to read/write trace, NULL if none.  If the same, then
      * trace information is in the same file, only one open is done.
@@ -501,12 +519,6 @@ handle_dev_fd_read(struct devio *io)
 				 port->tcp_to_dev.maxsize);
     }
 
-    if (port->dev_monitor != NULL) {
-	controller_write(port->dev_monitor,
-			 (char *) port->dev_to_tcp.buf,
-			 count);
-    }
-
     if (count < 0) {
 	if (errno == EAGAIN || errno == EWOULDBLOCK) {
 	    /* Nothing to read, just return. */
@@ -523,6 +535,30 @@ handle_dev_fd_read(struct devio *io)
 	return;
     }
 
+    if (port->dev_monitor != NULL) {
+	controller_write(port->dev_monitor,
+			 (char *) port->dev_to_tcp.buf,
+			 count);
+    }
+
+    if (port->closeon) {
+	int i;
+
+	for (i = 0; i < count; i++) {
+	    if (port->dev_to_tcp.buf[i] == port->closeon[port->closeon_pos]) {
+		port->closeon_pos++;
+		if (port->closeon_pos >= port->closeon_len) {
+		    port->close_on_output_done = true;
+		    /* Ignore everything after the closeon string */
+		    count = i + 1;
+		    break;
+		}
+	    } else {
+		port->closeon_pos = 0;
+	    }
+	}
+    }
+
     if (port->tr)
 	/* Do read tracing, ignore errors. */
 	do_trace(port, port->tr, port->dev_to_tcp.buf, count, SERIAL);
@@ -586,6 +622,9 @@ handle_dev_fd_read(struct devio *io)
 	    sel_set_fd_write_handler(ser2net_sel, port->tcpfd,
 				     SEL_FD_HANDLER_ENABLED);
 	    port->dev_to_tcp_state = PORT_WAITING_OUTPUT_CLEAR;
+	} else if (port->close_on_output_done) {
+	    shutdown_port(port, "closeon sequence found");
+	    port->close_on_output_done = false;
 	}
     }
 
@@ -822,6 +861,11 @@ tcp_fd_write(port_info_t *port, struct sbuf *buf)
 	sel_set_fd_write_handler(ser2net_sel, port->tcpfd,
 				 SEL_FD_HANDLER_DISABLED);
 	port->dev_to_tcp_state = PORT_WAITING_INPUT;
+
+	if (port->close_on_output_done) {
+	    shutdown_port(port, "closeon sequence found");
+	    port->close_on_output_done = false;
+	}
     }
 
     reset_timer(port);
@@ -842,9 +886,47 @@ static void
 handle_tcp_fd_except(int fd, void *data)
 {
     port_info_t *port = (port_info_t *) data;
+    int rv, val;
+    unsigned char c;
+    int cmd_pos;
 
-    syslog(LOG_ERR, "Select exception on port %s", port->portname);
-    shutdown_port(port, "tcp fd exception");
+    /* We should have urgent data, a DATA MARK in the stream.  Read
+       then urgent data (whose contents are irrelevant) then discard
+       user data until we find the DATA_MARK command. */
+
+    while ((rv = recv(fd, &c, 1, MSG_OOB)) > 0)
+	;
+    /* Ignore any errors, they are irrelevant. */
+
+    if (port->enabled != PORT_TELNET)
+	return;
+
+    /* Flush the data in the local and device queue. */
+    port->tcp_to_dev.cursize = 0;
+    val = 0;
+    port->io.f->flush(&port->io, &val);
+
+    /* Store it if we last got an IAC, and abort any current
+       telnet processing. */
+    cmd_pos = port->tn_data.telnet_cmd_pos;
+    if (cmd_pos != 1)
+	cmd_pos = 0;
+    port->tn_data.telnet_cmd_pos = 0;
+    port->tn_data.suboption_iac = 0;
+
+    while ((rv = read(fd, &c, 1)) > 0) {
+	if (cmd_pos == 1) {
+	    if (c == TN_DATA_MARK) {
+		/* Found it. */
+		if (port->telnet_brk_on_sync)
+		    port->io.f->send_break(&port->io);
+		break;
+	    }
+	    cmd_pos = 0;
+	} else if (c == TN_IAC) {
+	    cmd_pos = 1;
+	}
+    }
 }
 
 static void
@@ -871,7 +953,7 @@ telnet_cmd_handler(void *cb_data, unsigned char cmd)
 {
     port_info_t *port = cb_data;
 
-    if (cmd == TN_BREAK)
+    if ((cmd == TN_BREAK) || (port->telnet_brk_on_sync && cmd == TN_DATA_MARK))
 	port->io.f->send_break(&port->io);
 }
 
@@ -1631,6 +1713,8 @@ free_port(port_info_t *port)
 	free(port->openstr);
     if (port->closestr)
 	free(port->closestr);
+    if (port->closeon)
+	free(port->closeon);
     free(port);
 }
 
@@ -1812,6 +1896,7 @@ myconfig(void *data, struct absout *eout, const char *pos)
     port_info_t *port = data;
     enum str_type stype;
     char *s;
+    unsigned int len;
 
     if (strcmp(pos, "remctl") == 0) {
 	port->allow_2217 = 1;
@@ -1859,7 +1944,9 @@ myconfig(void *data, struct absout *eout, const char *pos)
 	/* get RS485 configuration. */
 	port->rs485conf = find_rs485conf(pos + 6);
 #endif
-    } else if ((s = find_str(pos, &stype))) {
+    } else if (strncmp(pos, "telnet_brk_on_sync", 3) == 0) {
+	port->telnet_brk_on_sync = 1;
+    } else if ((s = find_str(pos, &stype, &len))) {
 	/* It's a startup banner, signature or open/close string, it's
 	   already set. */
 	switch (stype) {
@@ -1867,6 +1954,7 @@ myconfig(void *data, struct absout *eout, const char *pos)
 	case SIGNATURE: port->signaturestr = s; break;
 	case OPENSTR: port->openstr = s; break;
 	case CLOSESTR: port->closestr = s; break;
+	case CLOSEON: port->closeon = s; port->closeon_len = len; break;
 	default: free(s);
 	}
     } else {
