@@ -20,6 +20,9 @@
 #ifndef READCONFIG
 #define READCONFIG
 
+/* Handle one line of configuration. */
+void handle_config_line(char *inbuf);
+
 /* Read the specified configuration file and call the routine to
    create the ports. */
 int readconfig(char *filename);
