@@ -34,6 +34,7 @@
 #include <unistd.h>
 #include <errno.h>
 
+#include "ser2net.h"
 #include "readconfig.h"
 #include "controller.h"
 #include "utils.h"
@@ -49,7 +50,6 @@ static int debug = 0;
 #ifdef USE_UUCP_LOCKING
 int uucp_locking_enabled = 1;
 #endif
-int cisco_ios_baud_rates = 0;
 
 selector_t *ser2net_sel;
 char *rfc2217_signature = "ser2net";
@@ -69,7 +69,7 @@ static char *help_string =
 #ifdef USE_UUCP_LOCKING
 "  -u - Disable UUCP locking\n"
 #endif
-"  -b - Do CISCO IOS baud-rate negotiation, instead of RFC2217\n"
+"  -b - unused (was Do CISCO IOS baud-rate negotiation, instead of RFC2217)\n"
 "  -v - print the program's version and exit\n"
 "  -s - specify a default signature for RFC2217 protocol\n";
 
@@ -103,6 +103,12 @@ reread_config(void)
 	    if (rv == CONTROLLER_INVALID_TCP_SPEC)
 		syslog(LOG_ERR, "Invalid control port specified: %s",
 		       config_port);
+	    else if (rv == CONTROLLER_OUT_OF_MEMORY)
+		syslog(LOG_ERR, "Out of memory opening control port: %s",
+		       config_port);
+	    else if (rv == CONTROLLER_CANT_OPEN_PORT)
+		syslog(LOG_ERR, "Can't open control port: %s",
+		       config_port);
 	    if (rv) {
 		syslog(LOG_ERR, "Control port is disabled");
 		free(config_port);
@@ -180,7 +186,6 @@ main(int argc, char *argv[])
 	    break;
 
 	case 'b':
-	    cisco_ios_baud_rates = 1;
 	    break;
 
 	case 'C':
