@@ -19,14 +19,16 @@
 
 /* This file holds basic utilities used by the ser2net program. */
 
+#include <stdlib.h>
 #include <string.h>
-#include <netdb.h>
-#include <sys/socket.h>
 #include <arpa/inet.h>
 #include <errno.h>
 #include <unistd.h>
+#include <fcntl.h>
 
+#include "ser2net.h"
 #include "utils.h"
+#include "selector.h"
 
 /* Scan for a positive integer, and return it.  Return -1 if the
    integer was invalid. */
@@ -64,32 +66,37 @@ scan_int(char *str)
    In the absence of a host specification, a wildcard address is used.
    The mandatory second part is the port number or a service name. */
 int
-scan_tcp_port(char *str, int domain,
-	      struct sockaddr_storage *addr, socklen_t *addr_len)
+scan_tcp_port(char *str, struct addrinfo **rai)
 {
-    char *strtok_data;
+    char *strtok_data, *strtok_buffer;
     char *ip;
     char *port;
     struct addrinfo hints, *ai;
 
-    memset(addr, 0, sizeof(*addr));
+    strtok_buffer = strdup(str);
+    if (!strtok_buffer)
+	return ENOMEM;
 
-    ip = strtok_r(str, ",", &strtok_data);
+    ip = strtok_r(strtok_buffer, ",", &strtok_data);
     port = strtok_r(NULL, "", &strtok_data);
     if (port == NULL) {
 	port = ip;
 	ip = NULL;
     }
-    
+
     memset(&hints, 0, sizeof(hints));
     hints.ai_flags = AI_PASSIVE;
-    hints.ai_family = domain;
-    if (getaddrinfo(ip, port, &hints, &ai))
-	return -1;
+    hints.ai_family = AF_UNSPEC;
+    hints.ai_socktype = SOCK_STREAM;
+    if (getaddrinfo(ip, port, &hints, &ai)) {
+	free(strtok_buffer);
+	return EINVAL;
+    }
 
-    memcpy(addr, ai->ai_addr, ai->ai_addrlen);
-    *addr_len = ai->ai_addrlen;
-    freeaddrinfo(ai);
+    free(strtok_buffer);
+    if (*rai)
+	freeaddrinfo(*rai);
+    *rai = ai;
     return 0;
 }
 
@@ -105,23 +112,75 @@ check_ipv6_only(int family, struct sockaddr *addr, int fd)
     }
 }
 
-int
-port_from_in_addr(int family, struct sockaddr *addr)
+int *
+open_socket(struct addrinfo *ai, void (*readhndlr)(int, void *), void *data,
+	    unsigned int *nr_fds)
 {
-    switch (family) {
-    case AF_INET6:
-	return ((struct sockaddr_in6 *) addr)->sin6_port;
+    struct addrinfo *rp;
+    int optval = 1;
+    int family = AF_INET6; /* Try IPV6 first, then IPV4. */
+    int *fds;
+    unsigned int curr_fd = 0;
+    unsigned int max_fds = 0;
 
-    case AF_INET:
-    default:
-	return ((struct sockaddr_in *) addr)->sin_port;
+    for (rp = ai; rp != NULL; rp = rp->ai_next)
+	max_fds++;
+
+    fds = malloc(sizeof(int) * max_fds);
+    if (!fds)
+	return NULL;
+
+  restart:
+    for (rp = ai; rp != NULL; rp = rp->ai_next) {
+	if (family != rp->ai_family)
+	    continue;
+
+	fds[curr_fd] = socket(rp->ai_family, rp->ai_socktype, rp->ai_protocol);
+	if (fds[curr_fd] == -1)
+	    continue;
+
+	if (fcntl(fds[curr_fd], F_SETFL, O_NONBLOCK) == -1)
+	    goto next;
+
+	if (setsockopt(fds[curr_fd], SOL_SOCKET, SO_REUSEADDR,
+		       (void *)&optval, sizeof(optval)) == -1)
+	    goto next;
+
+	check_ipv6_only(rp->ai_family, rp->ai_addr, fds[curr_fd]);
+
+	if (bind(fds[curr_fd], rp->ai_addr, rp->ai_addrlen) != 0)
+	    goto next;
+
+	if (listen(fds[curr_fd], 1) != 0)
+	    goto next;
+
+	sel_set_fd_handlers(ser2net_sel, fds[curr_fd], data,
+			    readhndlr, NULL, NULL);
+	sel_set_fd_read_handler(ser2net_sel, fds[curr_fd],
+				SEL_FD_HANDLER_ENABLED);
+	curr_fd++;
+	continue;
+
+      next:
+	close(fds[curr_fd]);
     }
+    if (family == AF_INET6) {
+	family = AF_INET;
+	goto restart;
+    }
+
+    if (curr_fd == 0) {
+	free(fds);
+	fds = NULL;
+    }
+    *nr_fds = curr_fd;
+    return fds;
 }
 
 int
 write_full(int fd, char *data, size_t count)
 {
-    size_t written;
+    ssize_t written;
 
  restart:
     while ((written = write(fd, data, count)) > 0) {
@@ -137,7 +196,7 @@ write_full(int fd, char *data, size_t count)
 }
 
 void
-write_ignore_fail(int fd, char *data, size_t count)
+write_ignore_fail(int fd, const char *data, size_t count)
 {
     ssize_t written;
 
